package cn.bunny.web.service.impl;

import cn.bunny.common.AuthContextUtil;
import cn.bunny.common.spzx.model.entity.h5.CartInfo;
import cn.bunny.common.spzx.model.entity.product.ProductSku;
import cn.bunny.web.service.CartService;
import com.alibaba.fastjson.JSON;
import jakarta.annotation.Resource;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
public class CartServiceImpl implements CartService {
    @Resource
    private RedisTemplate redisTemplate;

    // 定义key user:cart:userId
    private String getCartKey(Long userId) {
        return "user:cart:" + userId;
    }

    // 添加购物车
    @Override
    public void addToCart(Long skuId, Integer skuNum) {
        // 1.必须是登录状态，获取当前登录用户id，从ThreadLocal获取用户信息就可以了
        Long userId = AuthContextUtil.getUserInfo().getId();
        String cartKey = this.getCartKey(userId);

        // 2.因为购物车放到Redis里面，从Redis里面获取购物车数据，根据key值 + skuId获取(hash类型key+field)
        // hash类型 key：useId fileId：skuId value：sku信息CartInfo
        Object cartInfoObj = redisTemplate.opsForHash().get(cartKey, String.valueOf(skuId));

        // 3.如果购物车存在添加商品，把商品数量相加
        CartInfo cartInfo = null;
        if (cartInfoObj != null) {
            cartInfo = JSON.parseObject(cartInfoObj.toString(), CartInfo.class);
            // 数量相加
            cartInfo.setSkuNum(cartInfo.getSkuNum() + skuNum);
            // 购物车商品是选中状态
            cartInfo.setIsChecked(1);
            cartInfo.setUpdateTime(new Date());
        } else {
            // 4.如果购物车没有商品，直接商品添加购物车(添加到Redis里面)
            // 远程调用实现：通过nacos + openFeign 实现，根据skuId获取商品sku信息
            cartInfo = new CartInfo();
            // 远程调用实现：根据skuId获取商品sku信息
            ProductSku productSku = null;
            cartInfo.setCartPrice(productSku.getSalePrice());
            cartInfo.setSkuNum(skuNum);
            cartInfo.setSkuId(skuId);
            cartInfo.setUserId(userId);
            cartInfo.setImgUrl(productSku.getThumbImg());
            cartInfo.setSkuName(productSku.getSkuName());
            cartInfo.setIsChecked(1);
            cartInfo.setCreateTime(new Date());
            cartInfo.setUpdateTime(new Date());
        }

        redisTemplate.opsForHash().put(cartKey, String.valueOf(skuId), JSON.toJSONString(cartInfo));
    }
}
