package cn.bunny.web.product.service.imp;

import cn.bunny.common.spzx.model.dto.h5.ProductSkuDto;
import cn.bunny.common.spzx.model.entity.product.Product;
import cn.bunny.common.spzx.model.entity.product.ProductDetails;
import cn.bunny.common.spzx.model.entity.product.ProductSku;
import cn.bunny.common.spzx.model.vo.h5.ProductItemVo;
import cn.bunny.web.product.mapper.ProductDetailsMapper;
import cn.bunny.web.product.mapper.ProductMapper;
import cn.bunny.web.product.mapper.ProductSkuMapper;
import cn.bunny.web.product.service.ProductService;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import jakarta.annotation.Resource;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class ProductServiceImpl implements ProductService {
    @Resource
    private ProductSkuMapper productSkuMapper;
    @Resource
    private ProductMapper productMapper;
    @Resource
    private ProductDetailsMapper productDetailsMapper;
    @Resource
    private RedisTemplate redisTemplate;

    @Cacheable(value = "productSku", key = "'one'")
    @Override
    public List<ProductSku> selectProductSkuBySal() {
        // 1. 查询redis是否有一级分类
        Object productSkuJson = redisTemplate.opsForValue().get("productSku::one");
        String productSkuJsonString = JSON.toJSONString(productSkuJson);

        //  2. 如果存在一级分类直接返回即可
        if (!Objects.equals(productSkuJsonString, "null")) {
            return JSON.parseArray(productSkuJsonString);
        }
        // 3. 如果redis没有一级分类，查询数据库，把数据库内容返回，并且查询放到redis中
        // List<ProductSku> productSkuList = productSkuMapper.selectProductSkuBySale();
        // redisTemplate.opsForValue()
        //         .set("productSku::one", JSON.toJSON(productSkuList), 7, TimeUnit.DAYS);
        // return productSkuList;
        return productSkuMapper.selectProductSkuBySale();
    }

    // 分页查询
    @Override
    public PageInfo<ProductSku> findByPage(Integer page, Integer limit, ProductSkuDto productSkuDto) {
        PageHelper.startPage(page, page);
        List<ProductSku> productSkuList = productSkuMapper.findByPage(productSkuDto);
        return new PageInfo<>(productSkuList);
    }

    // 商品详情
    @Override
    public ProductItemVo itemBySkuId(Long skuId) {
        // 1.创建vo对象，用于封装最终数据
        ProductItemVo productItemVo = new ProductItemVo();
        // 2. 根据skuId获取sku信息
        ProductSku productSku = productSkuMapper.getById(skuId);
        // 3. 根据第二步获取sku，从sku中获取productId，获取商品数量
        Long productId = productSku.getProductId();
        Product product = productMapper.getById(productId);
        // 4.productId,获取商品详情skuId信息
        ProductDetails productDetails = productDetailsMapper.getByProductId(productId);
        // 5.封装map集合 == 商品规则对应商品skuId信息
        Map<String, Object> skuSpecValueMap = new HashMap<>();
        List<ProductSku> productSkuList = productSkuMapper.findByProductId(productId);
        productSkuList.forEach(item -> {
            skuSpecValueMap.put(item.getSkuSpec(), item.getId());
        });
        // 6.把需要封装数据到productItemVo中
        String imageUrls = productDetails.getImageUrls();
        String[] split = imageUrls.split(",");
        List<String> list = Arrays.asList(split);

        productItemVo.setProduct(product);
        productItemVo.setProductSku(productSku);
        productItemVo.setSkuSpecValueMap(skuSpecValueMap);
        productItemVo.setDetailsImageUrlList(list);
        productItemVo.setSliderUrlList(Arrays.asList(product.getSliderUrls().split(",")));
        productItemVo.setSpecValueList(JSON.parseArray(product.getSpecValue()));

        return productItemVo;
    }
}
