package cn.bunny.user.service.impl;

import cn.bunny.common.AuthContextUtil;
import cn.bunny.common.exception.BunnyException;
import cn.bunny.common.spzx.model.dto.h5.UserLoginDto;
import cn.bunny.common.spzx.model.dto.h5.UserRegisterDto;
import cn.bunny.common.spzx.model.entity.user.UserInfo;
import cn.bunny.common.spzx.model.vo.common.ResultCodeEnum;
import cn.bunny.common.spzx.model.vo.h5.UserInfoVo;
import cn.bunny.user.mapper.UserinfoMapper;
import cn.bunny.user.service.UserinfoService;
import com.alibaba.fastjson.JSON;
import jakarta.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class UserinfoServiceImpl implements UserinfoService {
    @Resource
    private RedisTemplate redisTemplate;
    @Resource
    private UserinfoMapper userinfoMapper;

    // 获取当前登录用户信息
    @Override
    public UserInfoVo getCurrentUserInfo(String token) {
/*         // 从Redis根据token获取用户信息
        String userJson = (String) redisTemplate.opsForValue().get("user:spzx" + token);
        if (!StringUtils.hasText(userJson)) {
            throw new BunnyException(ResultCodeEnum.LOGIN_ERROR);
        }

        UserInfo userinfo = JSON.parseObject(userJson, UserInfo.class);
        assert userinfo != null;
        UserInfoVo userInfoVo = new UserInfoVo();
        BeanUtils.copyProperties(userinfo, userInfoVo);

        return userInfoVo; */
        UserInfo userInfo = AuthContextUtil.getUserInfo();
        UserInfoVo userInfoVo = new UserInfoVo();
        BeanUtils.copyProperties(userInfo, userInfoVo);
        return userInfoVo;
    }

    // 会员注册
    @Override
    public void register(UserRegisterDto userRegisterDto) {
        // 1. userLoginDto获取数据
        String username = userRegisterDto.getUsername();
        String password = userRegisterDto.getPassword();
        String nickName = userRegisterDto.getNickName();
        String code = userRegisterDto.getCode();
        // 2. 从Redis获取发送验证码
        String redisCode = (String) redisTemplate.opsForValue().get(username);
        // 3. 获取输入验证码，进行对比
        if (!redisCode.equals(code)) {
            throw new BunnyException(ResultCodeEnum.VALIDATECODE_ERROR);
        }
        // 4. 校验用户名不重复
        UserInfo userInfo = userinfoMapper.getByUsername(username);
        if (userInfo != null) {
            throw new BunnyException(ResultCodeEnum.USER_NAME_IS_EXISTS);
        }
        // 5. 封装添加数据，调用方法调用添加数据
        userInfo = new UserInfo();
        userInfo.setUsername(username);
        userInfo.setNickName(nickName);
        userInfo.setPassword(DigestUtils.md5DigestAsHex(password.getBytes()));
        userInfo.setPhone(username);
        userInfo.setStatus(1);
        userInfo.setSex(0);
        userInfo.setAvatar("http://thirdwx.qlogo.cn/mmopen/vi_32/DYAIOgq83eoj0hHXhgJNOTSOFsS4uZs8x1ConecaVOB8eIl115xmJZcT4oCicvia7wMEufibKtTLqiaJeanU2Lpg3w/132");
        userinfoMapper.save(userInfo);
        // 6. 从Redis中删除发送验证码
        redisTemplate.delete(username);
    }

    // 会员登录
    @Override
    public String login(UserLoginDto userLoginDto) {
        // 1. dto获取用户名和密码
        String username = userLoginDto.getUsername();
        String password = userLoginDto.getPassword();
        // 2. 根据用户名查询数据库，得到用户信息
        UserInfo userInfo = userinfoMapper.selectByUsername(username);
        // 3. 比较密码是否一致
        String database_password = userInfo.getPassword();
        String md5_password = DigestUtils.md5DigestAsHex(password.getBytes());
        if (!database_password.equals(md5_password)) {
            throw new BunnyException(ResultCodeEnum.LOGIN_ERROR);
        }
        // 4. 校验用户名是否禁用
        if (userInfo.getStatus() == 0) {
            throw new BunnyException(ResultCodeEnum.ACCOUNT_STOP);
        }
        // 5. 生成token
        String token = UUID.randomUUID().toString().replaceAll("-", "");
        // 6. 把用户名信息放到redis中
        redisTemplate.opsForValue().set("user:spzx" + token,
                JSON.toJSONString(userInfo), 30, TimeUnit.DAYS);
        // 7. 返回token
        return token;
    }
}
